{-----------------------------------------------------------------------------
   Unit Name: interunit
   Purpose  : Show how to use Delphi's interfaces
   Author   : Vesa Lappalainen
   Date     : 8.11.1999
   Changed  : 25.11.2001/vl
     + VCL/CLX compiling
     4.8.2004 / mn
       added interface querying example
   Todo     :
-----------------------------------------------------------------------------}

unit interunit;

interface

uses
  SysUtils,
{$ifdef CLX}
  QGraphics, QControls, QForms, QDialogs, QStdCtrls,
{$else}
  Windows, Messages,
  Graphics, Controls, Forms, Dialogs, StdCtrls,
{$endif}
  Classes;

type
  IAsString = interface
  //MN: GUID. tytyy olla, jotta rajapinnan kyselyfunktiot toimivat
  ['{3108850B-8281-4DF2-BAE4-4ED729B02A40}']
    function GetAsString: string;
    procedure SetAsString(const Value: string);
    property AsString:string read GetAsString write SetAsString;
  end;

  TFormInterfaceDemo = class(TForm)
    procedure FormCreate(Sender: TObject);
  private
    list : Array of IAsString;
    procedure ButtonClick(Sender: TObject);
  public
  end;


  TStButton = class(TButton,IAsString)
  private
    function GetAsString: string;
    procedure SetAsString(const Value: string);
  published
    property AsString:string read GetAsString write SetAsString;
  end;

  TStEdit = class(TEdit,IAsString)
  private
    function GetAsString: string;
    procedure SetAsString(const Value: string);
  public
    destructor Destroy; override;
  published
    property AsString:string read GetAsString write SetAsString;
  end;


var
  FormInterfaceDemo: TFormInterfaceDemo;

implementation

{$ifdef CLX}
{$R *.dfm}
{$else}
{$R *.dfm}
{$endif}


{ TStButton }

function TStButton.GetAsString: string;
begin
  Result := Caption;
end;

procedure TStButton.SetAsString(const Value: string);
begin
  Caption := Value;
end;

{ TStEdit }

destructor TStEdit.Destroy;
begin
  inherited;
end;

function TStEdit.GetAsString: string;
begin
  Result := Text;
end;

procedure TStEdit.SetAsString(const Value: string);
begin
  Text := Value;
end;

{ TFormInterfaceDemo }

procedure TFormInterfaceDemo.FormCreate(Sender: TObject);
var b : TStButton; e : TStEdit; ed:TEdit; s:IAsString;
    i : integer;
begin
  b := TStButton.Create(self);
  b.Parent := self;
  b.OnClick := ButtonClick;

  ed := TStEdit.Create(self);
  ed.Parent := self;
  ed.Top := 50;

  //MN: testataan rajapinnan kysely

  //MN: ei toimi
  //  if (ed is IAsString) then e:=ed as IAsString;
  if Supports(ed,IAsString) then s:=ed as IAsString;

  // MN: tai:
  //ed.GetInterface(IAsString,s); // palauttaa true, jos tukee rajapintaa

  SetLength(list,2);
  list[0] := b;
  list[1] := s;// e;

  for i:=0 to High(list) do
    list[i].AsString := 'Object ' + IntToStr(i);

end;

procedure TFormInterfaceDemo.ButtonClick(Sender: TObject);
var i : integer;
begin
  for i:=0 to High(list) do list[i].AsString := 'Kissa';
end;


end.
